'''
Holds aplication wide configuration parameters
'''
import json
import logging
import copy
from argparse import ArgumentParser
from app_util import connections

VALID_PROFILES = ['dev', 'stage', 'prod']
LOGGER_FMT = "[%(levelname)s] [%(asctime)s] [%(name)s] - %(message)s"
LOGGER_DATE_FMT = "%m/%d/%Y %H:%M:%S"

def load_config(environment=None, config_file='config.json', argparser=None):
    '''
    Automatically loads configuration using the config file and argparser.
    First loads from the file, then command line arguments are loaded over
    the config file arguments.

    Whether or not an argparser is provided, the following arguments will be used/overidden:
    parser.add_argument('--environment', '-e', dest='environment', default='dev')
    parser.add_argument('--debug', '-d', dest='debug', action='store_true')

    returns the Namepsace object, as well as auto configured Connections object
    '''
    parser = ArgumentParser(parents=[argparser], conflict_handler='resolve') \
            if argparser else ArgumentParser()
    parser.add_argument('--environment', '-e', dest='environment', default='dev',
                        choices=VALID_PROFILES)
    parser.add_argument('--debug', '-d', dest='debug', action='store_true')
    conf, _ = parser.parse_known_args()
    environment = environment if environment in VALID_PROFILES else conf.environment
    profile_data = copy.deepcopy(conf)

    # Enable debug logging as soon as possible
    logging.basicConfig(level=logging.DEBUG if getattr(profile_data, 'debug', False) \
                                            else logging.INFO,
                        format=LOGGER_FMT, datefmt=LOGGER_DATE_FMT)
    log = logging.getLogger(__name__)
    log.debug("Debug logging enabled")

    with open(config_file) as configuration:
        try:
            config_data = json.load(configuration)
            if not isinstance(config_data, list):
                log.warning("File '%s' is not in the correct format, " \
                               + " expected a list of maps.", config_file)
            else:
                for profile in config_data:
                    if profile.get('environment') == environment:
                        profile_data.__dict__.update(profile)

        except json.decoder.JSONDecodeError:
            log.error("Could not load config file %s, check your JSON", config_file)

    profile_data.__dict__.update(conf.__dict__)
    log.info("Loaded environment: %s ", environment)

    return profile_data, connections.Connections(**profile_data.__dict__)



"""Holds utility functions useful in a variety of situations"""
import logging

LOGGER = logging.getLogger(__name__)

def find_function(func_path):
    '''Searches for and returns a function if it exists'''
    LOGGER.debug("Searching for function %s", func_path)
    paths = func_path.split(':')
    module = __import__(''.join(paths[:-1]), fromlist=['.'.join(paths[1:])])
    func = getattr(module, paths[-1])

    return func


class Email(object):
    '''A convenience object to create an email payload'''

    def __init__(self, subject, tos, ccs=None, bccs=None):
        '''
        Initalize an email with the recipients and subject. Content
        is not added here since its construction is a bit more complex.

        subject: This goes in the subject field
        tos, ccs, bccs: Lists of email addresses
        '''
        self.tos = tos
        self.ccs = ccs
        self.bccs = bccs
        self.subject = subject
        self.payload = {}

    def set_content(self, template='generic', **kwargs):
        '''
        Sets the content portion of the email. HTML is allowed.

        template: Email template to use. All templates have the following attributes,
                  the difference lies in how the "content" paramter is interpreted

        Recognized arguments for kwargs are:
        title: This goes at the top of the email in a large font
        header: This is a normal paragraph that starts the first portion.
        content: The template specific information goes here.
        footer: Like the header, except it comes after the content.
        '''
        self.payload = kwargs
        self.payload['template'] = template

    def set_recipients(self, tos=None, ccs=None, bccs=None):
        '''
        Sets the recipients for the email.

        Pass an empty list for recipient type to remove all recipients.
        '''
        self.tos = tos if tos != None else self.tos
        self.ccs = ccs if ccs != None else self.ccs
        self.bccs = bccs if bccs != None else self.bccs



"""
Holds the configuration information for this software
"""

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

setup(name='app-utilities',
      packages=['app_util'],
      version='0.1.6',
      description='A utility to fast-track application configurations',
      author='Philip Whiting',
      author_email='phwhitin@cisco.com',
      url='https://github.com/Himself12794/app-utilities',
      download_url='https://github.com/Himself12794/app-utilities/archive/v0.1.6.tar.gz',
      keywords=['configuration', 'utility'],
      install_requires=['pymongo>=3.0.0', 'requests>=2.0.0']
     )



'''
Holds connection information for the project, as well as handles sending messages to Spark
@author Philip Whiting (phwhitin)
'''
import logging
from pymongo import MongoClient
import requests
LOGGER = logging.getLogger(__name__)

API_BASE = '/code/cda/notification/v1/publish'

SS1 = 'SCRAM-SHA-1'

JSON_HEADERS = {'Content-Type': 'application/json;charset=utf-8'}

class Connections(object):
    '''Configurable connections interface for interaction with remote resources'''

    def __init__(self, **kwargs):
        '''Init function'''
        self.environment = kwargs.get('environment')
        self.notification_api_url = kwargs.get('notification_api_url')
        self.mongo_db_username = kwargs.get('mongo_db_username')
        self.mongo_db_password = kwargs.get('mongo_db_password')
        self.mongo_db_prefix = kwargs.get('mongo_db_prefix')
        self.default_spark_room = kwargs.get('default_spark_room')
        self.spark_token = kwargs.get('spark_token')
        self._mongos = {}
        self._auths = {}

    def publish_to_spark(self, msg, room=None):
        '''Publishes a message to the spark room via the notification API'''
        room = room or self.default_spark_room
        payload = _generate_api_payload(msg)
        destination = payload['destination']
        destination['roomId'] = room
        destination['bearerToken'] = self.spark_token
        return self._publish_payload(payload)

    def publish_email_obj(self, email, sender):
        '''Sends an email using the email object'''
        return self.publish_email(email.payload, sender, email.subject,
                                  email.tos, email.ccs, email.bccs)

    def publish_email(self, data, sender, subject='', tos=None, ccs=None, bccs=None):
        '''Sends an email via the notification API'''
        payload = _generate_api_payload(data, message_type='EMAIL')
        payload['subject'] = subject
        payload['sender'] = sender
        destination = payload['destination']
        destination['emailAddresses'] = tos or []
        destination['ccEmailAddresses'] = ccs or []
        destination['bccEmailAddresses'] = bccs or []

        return self._publish_payload(payload)

    def _publish_payload(self, payload):
        '''Publishes the payload to the notification API'''
        return requests.post(self.notification_api_url + API_BASE,
                             headers=JSON_HEADERS, json=payload, verify=self.environment != 'dev')

    def get_mongo(self, num, *auth_dbs):
        '''
        Opens and returns a Mongo Client for dft-mongo-num for the specified auth dbs.
        This client is cached to prevent multiple connections
        '''
        host = self._get_host(num)
        client = self._mongos.get(host, MongoClient(host, 18000))
        auths = self._auths.get(host, [])
        for auth_db in auth_dbs:
            if not auth_db in auths:
                client[auth_db].authenticate(self.mongo_db_username, self.mongo_db_password,
                                             mechanism=SS1)
                auths.append(auth_db)

        self._auths[host] = auths
        if not host in self._mongos:
            self._mongos[host] = client

        return client

    def _get_host(self, num):
        '''Determines host based on config'''
        return {
            'prod': self.mongo_db_prefix + ('' if num <= 0 else '-' + str(num))
        }.get(self.environment, self.mongo_db_prefix + str(num + 1))

def _generate_api_payload(data, message_type='SPARK'):
    '''Generates the payload for a call to the notification API'''
    return {
        'data': data,
        'destination': {
            'type': message_type
        }
    }

def disable_warnings():
    '''Disables the SSL warnings about indesurec requests'''
    from requests.packages.urllib3.exceptions import InsecureRequestWarning
    requests.packages.urllib3.disable_warnings(InsecureRequestWarning)
